//###################################################################
// File: ColoursPWM.c
//
// Ein kleines Farbenspiel mit einer Dreifarben-LED
//
// Wichtig: Auf keinen Fall die Dinger mit dem glasklaren Gehuse
// benutzen. Nur mit dem diffusen Gehuse sieht das ganze gut aus.
//
// Die beiden PWM-Module im ATiny26 werden benutzt um die Helligkeit
// der LED's zu steuern.
//
// Die eine blaue LED wird an PORTB0 angeschlossen. Die zweite an
// PORTB2. Die grne LED wird an PORTB3 angeschlossen und die rote
// LED an PORTB1.
//
// Zuerst werden rot und grn gemischt. PWM-Modul A und B laufen dabei
// gegeneinander. Grn wird immer dunkler und rot immer heller. Die
// invertierten Ausgnge der PWM werden nicht benutzt und die blauen
// LED's sind abgeschaltet.
//
// Danach luft nur PWM-Modul A mit invertiertem Ausgang. Rot wird immer
// dunkler und blau immer heller.
//
// Dann luft nur PWM-Modul B mit invertiertem Ausgang. Blau wird immer
// dunkler und grn immer heller.
//
// Da das Timing nicht kritisch ist kann man ruhig den internen
// RC-Osci benutzen. Am besten auf 2MHz einstellen. ber PORTA kann
// man die Geschwindigkeit einstellen. PORTA offen ist am langsamsten.
// PORTA0 auf Masse 1/4 der Geschwindigkeit, PORTA1 auf Masse 1/16.
// 
//###################################################################
// Compiler: WINAVR V3.3
//###################################################################
// Processor: ATTiny26
//
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
// Last change: 26.09.2003
//###################################################################

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

#define PWM_RED_ON()  TCCR1A |=(1<<PWM1A | 1<<COM1A1);
#define PWM_RED_OFF() TCCR1A&=~(1<<PWM1A | 1<<COM1A1);

#define PWM_RED_BLUE_ON()  TCCR1A |=(1<<PWM1A | 1<<COM1A0);
#define PWM_RED_BLUE_OFF() TCCR1A&=~(1<<PWM1A | 1<<COM1A0);

#define PWM_GREEN_ON()  TCCR1A |=(1<<PWM1B | 1<<COM1B1);
#define PWM_GREEN_OFF() TCCR1A&=~(1<<PWM1B | 1<<COM1B1);

#define PWM_GREEN_BLUE_ON()  TCCR1A |=(1<<PWM1B | 1<<COM1B0);
#define PWM_GREEN_BLUE_OFF() TCCR1A&=~(1<<PWM1B | 1<<COM1B0);

#define PWM_VALUE	199
#define UP_VALUE	0
#define DOWN_VALUE	PWM_VALUE

#define BLUE_ON()  sbi(PORTB,0);
#define BLUE_OFF() cbi(PORTB,0);

#define BLUE2_ON()  sbi(PORTB,2);
#define BLUE2_OFF() cbi(PORTB,2);

#define GREEN_ON()  sbi(PORTB,3);
#define GREEN_OFF() cbi(PORTB,3);

#define RED_ON()  sbi(PORTB,1);
#define RED_OFF() cbi(PORTB,1);

#define STOP	0
#define UP	1
#define DOWN	2

//"volatile" fr alle Variablen die in Interrupt's gendert werden
unsigned char volatile redcount;
unsigned char volatile reddir;
unsigned char volatile greencount;
unsigned char volatile greendir;
unsigned char volatile bluecount;
unsigned char volatile bluecount2;

//###################################################################################
SIGNAL(SIG_OVERFLOW0)        /* signal handler for tcnt0 overflow interrupt */
//###################################################################################
{
 if(reddir==DOWN) { redcount--; if(redcount==0)         reddir=STOP; OCR1A=redcount; }
 if(reddir==UP)   { redcount++; if(redcount==PWM_VALUE) reddir=STOP; OCR1A=redcount; }
 
 if(greendir==DOWN) { greencount--; if(greencount==0)         greendir=STOP; OCR1B=greencount; }
 if(greendir==UP)   { greencount++; if(greencount==PWM_VALUE) greendir=STOP; OCR1B=greencount; }

 if(PINA==0xFF) TCCR0=0x05; 
 if(PINA==0xFE) TCCR0=0x04; 
 if(PINA==0xFD) TCCR0=0x03; 
}

//###################################################################################
/* Hauptprogramm */
int main(void)
//###################################################################################
{
//1 = Ausgang
//0 = Eingang
 DDRA=0x00;  	// set io-pins on PortA
 PORTA=0xFF;    // Pullups 	

 DDRB=0x0F;  	// set io-pins on PortB
 PORTB=0; 	

//Timer0
 TCCR0=0x05; 

//Timer1
 TCCR1A=0; 
 TCCR1B=0x05;        //Timer1 Start
 TIMSK=(1<<TOIE0);
 sei();              //enable interrupts

 redcount=0;
 greencount=0;
 bluecount=0;
 bluecount2=0;
 greendir=STOP;
 reddir=STOP;

 OCR1C=PWM_VALUE;
 for(;;) // Anfang Programm Endlosschleife
  {
//bergang grn zu rot
   redcount=UP_VALUE;
   greencount=DOWN_VALUE;
   OCR1A=redcount;
   OCR1B=greencount;
   reddir=UP;
   greendir=DOWN;
   PWM_RED_ON();
   PWM_GREEN_ON();
   while(greendir!=STOP);
   PWM_RED_OFF();
   PWM_GREEN_OFF();

//bergang rot zu blau
   PWM_RED_BLUE_ON();
   redcount=DOWN_VALUE;
   OCR1A=redcount;
   reddir=DOWN;
   while(reddir!=STOP);
   PWM_RED_BLUE_OFF();

//bergang blau zu grn
   PWM_GREEN_BLUE_ON();
   greencount=UP_VALUE;
   OCR1B=greencount;
   greendir=UP;
   while(greendir!=STOP);
   PWM_GREEN_BLUE_OFF();

  } // Ende Programm Endlosschleife
}
